InteractiveControls = {};

function InteractiveControls.prerequisitesPresent(specializations)
    return true;
end;

function InteractiveControls:load(xmlFile)

	self.toggleSupport = SpecializationUtil.callSpecializationsFunction("toggleSupport");
	self.puntepos = 2
	-- Punte
	self.punte = {};
	
	self.punte.interne = {};
	self.punte.interne.animation = getXMLString(xmlFile, "vehicle.punte#interneAnimation");
	if self.punte.interne.animation then 
		self.punte.rangeIndex = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.punte#RangeIndex"));
		self.punte.inRange = false;
		self.punte.interne.isOpen = true;
		self.punte.interne.dirtyFlag = self:getNextDirtyFlag();
	end;
	
	self.punte.esterne = {};
	self.punte.esterne.animation = getXMLString(xmlFile, "vehicle.punte#esterneAnimation");
	if self.punte.esterne.animation then 
		self.punte.rangeIndex = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.punte#RangeIndex"));
		self.punte.inRange = false;
		self.punte.esterne.isOpen = true;
		self.punte.esterne.dirtyFlag = self:getNextDirtyFlag();
	end;
	
	self.punte.centrali = {};
	self.punte.centrali.animation = getXMLString(xmlFile, "vehicle.punte#centraliAnimation");
	if self.punte.centrali.animation then 
		self.punte.rangeIndex = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.punte#RangeIndex"));
		self.punte.inRange = false;
		self.punte.centrali.isOpen = true;
		self.punte.centrali.dirtyFlag = self:getNextDirtyFlag();
	end;
	
	-- Hydraulic hoses
	self.hydraulicHoses = {};
	self.hydraulicHoses.attached = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.hydraulicHoses#attached"));
	self.hydraulicHoses.detached = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.hydraulicHoses#detached"));
	
	-- Support
	self.manualSupportAnimation = getXMLString(xmlFile, "vehicle.manualSupport#animationName");
	self.supportRangeIndex = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.manualSupport#RangeIndex"));
	self.isInRangeSupport = false;
	self.isSupport = false;
end;

function InteractiveControls:delete()

end;

function InteractiveControls:readStream(streamId, connection)
	local isSupport = streamReadBool(streamId);
	self:toggleSupport(isSupport, true);

	self.punte.interne.isOpen = streamReadBool(streamId);
	self.punte.esterne.isOpen = streamReadBool(streamId);
	self.punte.centrali.isOpen = streamReadBool(streamId);
	
	if self.punte.interne.animation ~= nil and self.playAnimation ~= nil then
		if self.punte.interne.isOpen then
			self:playAnimation(self.punte.interne.animation, 1, nil, true);
		end;
	end;
	
	if self.punte.esterne.animation ~= nil and self.playAnimation ~= nil then
		if self.punte.esterne.isOpen then
			self:playAnimation(self.punte.esterne.animation, 1, nil, true);
		end;
	end;
	
	if self.punte.centrali.animation ~= nil and self.playAnimation ~= nil then
		if self.punte.centrali.isOpen then
			self:playAnimation(self.punte.centrali.animation, 1, nil, true);
		end;
	end;
end;

function InteractiveControls:writeStream(streamId, connection)
	streamWriteBool(streamId, self.isSupport);
	streamWriteBool(streamId, self.punte.interne.isOpen);
	streamWriteBool(streamId, self.punte.esterne.isOpen);
	streamWriteBool(streamId, self.punte.centrali.isOpen);
end;

function InteractiveControls:readUpdateStream(streamId, timestamp, connection)
    if connection:getIsServer() then
        local updateInternePanel = streamReadBool(streamId);
        if updateInternePanel then
			self.punte.interne.isOpen = streamReadBool(streamId);
        end;
        local updateEsternePanel = streamReadBool(streamId);
        if updateEsternePanel then
			self.punte.esterne.isOpen = streamReadBool(streamId);
        end;
        local updateCentraliDoor = streamReadBool(streamId);
        if updateCentraliPanel then
			self.punte.centrali.isOpen = streamReadBool(streamId);
        end;
    end;
end;

function InteractiveControls:writeUpdateStream(streamId, connection, dirtyMask)
    if not connection:getIsServer() then
        if bitAND(dirtyMask, self.punte.interne.dirtyFlag) ~= 0 then
            streamWriteBool(streamId, true);
            streamWriteBool(streamId, self.punte.interne.isOpen);
        else
            streamWriteBool(streamId, false);
        end;
        if bitAND(dirtyMask, self.punte.esterne.dirtyFlag) ~= 0 then
            streamWriteBool(streamId, true);
            streamWriteBool(streamId, self.punte.esterne.isOpen);
        else
            streamWriteBool(streamId, false);
        end;
        if bitAND(dirtyMask, self.punte.centrali.dirtyFlag) ~= 0 then
            streamWriteBool(streamId, true);
            streamWriteBool(streamId, self.punte.centrali.isOpen);
        else
            streamWriteBool(streamId, false);
        end;
    end;
end;

function InteractiveControls:mouseEvent(posX, posY, isDown, isUp, button)
end;

function InteractiveControls:keyEvent(unicode, sym, modifier, isDown)
end;

function InteractiveControls:update(dt)

	if self.supportRangeIndex then
		-- Manage key events for inrange support --
		if self.isInRangeSupport then
			if InputBinding.hasEvent(InputBinding.MODE) then
				if self.isSupport then
					self:toggleSupport(false);
				else
					self:toggleSupport(true);
				end;
			end;
			-- Display key when in range --
			if self.isSupport then
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("ATTACH_SUPPORT")), InputBinding.MODE);
			else
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("DETACH_SUPPORT")), InputBinding.MODE);
			end;
		end;
	end;

	if self.isSupport then
	
	if self.punte.rangeIndex then
		-- Manage key events for inrange punte --
		if self.punte.inRange then
			if InputBinding.hasEvent(InputBinding.MODE1) then
				if self.punte.interne.isOpen then
					if self.punte.interne.animation ~= nil and self.playAnimation ~= nil then
						self:playAnimation(self.punte.interne.animation, -1, nil);
						self.puntepos = 1;
					end;
				else
					if self.punte.interne.animation ~= nil and self.playAnimation ~= nil then
						self:playAnimation(self.punte.interne.animation, 1, nil);
					end;
				end;
				self.punte.interne.isOpen = self.punte.interne.isOpen;		
				self:raiseDirtyFlags(self.punte.interne.dirtyFlag);
			end;
			-- Display key when in range --
			if self.punte.interne.isOpen then
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("MODE1")), InputBinding.MODE1);
			else
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("ISBI")), InputBinding.MODE1);
			end;

			if InputBinding.hasEvent(InputBinding.MODE2) then
				if self.punte.centrali.isOpen then
					if self.punte.centrali.animation ~= nil and self.playAnimation ~= nil then
						self:playAnimation(self.punte.centrali.animation, -1, nil);
						self.puntepos = 2;
					end;
				else
					if self.punte.centrali.animation ~= nil and self.playAnimation ~= nil then
						self:playAnimation(self.punte.centrali.animation, 1, nil);
					end;
				end;
				self.punte.centrali.isOpen = self.punte.centrali.isOpen;
				self:raiseDirtyFlags(self.punte.centrali.dirtyFlag);
			end;
			-- Display key when in range --
			if self.punte.centrali.isOpen then
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("MODE2")), InputBinding.MODE2);
			else
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("ISBC")), InputBinding.MODE2);
			end;

			if InputBinding.hasEvent(InputBinding.MODE3) then
				if self.punte.esterne.isOpen then
					if self.punte.esterne.animation ~= nil and self.playAnimation ~= nil then
						self:playAnimation(self.punte.esterne.animation, -1, nil);
						self.puntepos = 3;
					end;
				else
					if self.punte.esterne.animation ~= nil and self.playAnimation ~= nil then
						self:playAnimation(self.punte.esterne.animation, 1, nil);
					end;
				end;
				self.punte.esterne.isOpen = self.punte.esterne.isOpen;
				self:raiseDirtyFlags(self.punte.esterne.dirtyFlag);
			end;
			-- Display key when in range --
			if self.punte.esterne.isOpen then
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("MODE3")), InputBinding.MODE3);
			else
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("ISBE")), InputBinding.MODE3);
			end;
			-- Display key when in range --
			if self.isSupport then
				if self.puntepos ~= 2 and self.puntepos ~= 3 then
					g_currentMission:addExtraPrintText(g_i18n:getText("ISBI"));
				end;
				if self.puntepos ~= 1 and self.puntepos ~= 3 then
					g_currentMission:addExtraPrintText(g_i18n:getText("ISBC"));
				end;
				if self.puntepos ~= 1 and self.puntepos ~= 2 then
					g_currentMission:addExtraPrintText(g_i18n:getText("ISBE"));
				end;
			end;
		end;
	end;
	end;
	
	if self:getIsActive() then		
		if self.isClient and self:getIsActiveForInput(false) and not self:hasInputConflictWithSelection() then
			-- Show warning about the support being down
			if self.isSupport then
				g_currentMission:addWarning(string.format(g_i18n:getText("SUPPORT_WARNING")), 0.018, 0.033);
			end;
		end;
	end;
end;

function InteractiveControls:updateTick(dt)
	if self.supportRangeIndex then
		if g_currentMission.player ~= nil then
			-- Getting the distance between the player and the support
			local nearestDistance = 2.0; --max distance allowed
			local px, py, pz = getWorldTranslation(self.supportRangeIndex);
			local vx, vy, vz = getWorldTranslation(g_currentMission.player.rootNode);
			local distance = Utils.vector3Length(px-vx, py-vy, pz-vz);
			if distance < nearestDistance then
				self.isInRangeSupport = true;
			else
				self.isInRangeSupport = false;
			end;
		end;
	else
		self.isInRangeSupport = false;
	end;
	
	if self.punte.rangeIndex then
		if g_currentMission.player ~= nil then
			-- Getting the distance between the player and the punte
			local nearestDistance = 2.0; --max distance allowed
			local px, py, pz = getWorldTranslation(self.punte.rangeIndex);
			local vx, vy, vz = getWorldTranslation(g_currentMission.player.rootNode);
			local distance = Utils.vector3Length(px-vx, py-vy, pz-vz);
			if distance < nearestDistance then
				self.punte.inRange = true;
			else
				self.punte.inRange = false;
			end;
		end;
	else
		self.punte.inRange = false;
	end;

end;

function InteractiveControls:draw()
	if self:getIsActiveForInput(true) then
		g_currentMission:addExtraPrintText(g_i18n:getText("Help"));
	end;
end;

function InteractiveControls:onAttach(attacherVehicle)
	setVisibility(self.hydraulicHoses.attached, true);
	setVisibility(self.hydraulicHoses.detached, false);
	self.attacherVehicle = attacherVehicle;
end;

function InteractiveControls:onDetach()
	setVisibility(self.hydraulicHoses.attached, false);
	setVisibility(self.hydraulicHoses.detached, true);
	self.attacherVehicle = nil;
end;

function InteractiveControls:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	local isSupport = Utils.getNoNil(getXMLBool(xmlFile, key .. "#isSupport"), false);
	self:toggleSupport(isSupport);
    return BaseMission.VEHICLE_LOAD_OK;
end;

function InteractiveControls:getSaveAttributesAndNodes(nodeIdent)

end;

function InteractiveControls:toggleSupport(isSupport, noEventSend)
	ToggleSupportEvent.sendEvent(self, isSupport, noEventSend);	
	if not isSupport then
		if self.manualSupportAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.manualSupportAnimation, -1, nil, true);
		end;
	else
		if self.manualSupportAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.manualSupportAnimation, 1, nil, true);
		end;
	end;
	self.isSupport = isSupport;
end;

-- Support event
ToggleSupportEvent = {};
ToggleSupportEvent_mt = Class(ToggleSupportEvent, Event);
InitEventClass(ToggleSupportEvent, "ToggleSupportEvent");

function ToggleSupportEvent:emptyNew()
    local self = Event:new(ToggleSupportEvent_mt);
    return self;
end;

function ToggleSupportEvent:new(vehicle, isSupport)
    local self = ToggleSupportEvent:emptyNew()
    self.vehicle = vehicle;
	self.isSupport = isSupport;
    return self;
end;

function ToggleSupportEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
	self.vehicle = networkGetObject(id);
	self.isSupport = streamReadBool(streamId);
    self:run(connection);
end;

function ToggleSupportEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));	
	streamWriteBool(streamId, self.isSupport);
end;

function ToggleSupportEvent:run(connection)
	self.vehicle:toggleSupport(self.isSupport, true);
end;

function ToggleSupportEvent.sendEvent(vehicle, isSupport, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(ToggleSupportEvent:new(vehicle, isSupport), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(ToggleSupportEvent:new(vehicle, isSupport));
		end;
	end;
end;